@@ -57,6 +57,7 @@ bool FourSuite::testAddOrRemoveBaseHelper(const char * ref, const char * base,
 	if (res != 0) {
 		uriFreeUriMembersA(&baseUri);
 		uriFreeUriMembersA(&relUri);
+		uriFreeUriMembersA(&expectedUri);
 		return false;
 	}
 
@@ -71,6 +72,8 @@ bool FourSuite::testAddOrRemoveBaseHelper(const char * ref, const char * base,
 	if (res != 0) {
 		uriFreeUriMembersA(&baseUri);
 		uriFreeUriMembersA(&relUri);
+		uriFreeUriMembersA(&expectedUri);
+		uriFreeUriMembersA(&transformedUri);
 		return false;
 	}
 
@@ -86,13 +89,14 @@ bool FourSuite::testAddOrRemoveBaseHelper(const char * ref, const char * base,
 	uriFreeUriMembersA(&baseUri);
 	uriFreeUriMembersA(&relUri);
 	uriFreeUriMembersA(&expectedUri);
+	uriFreeUriMembersA(&transformedUri);
 	return equal;
 }
 
 
 
 void FourSuite::absolutize_test_cases() {
-	char * BASE_URI[] = {
+	const char * const BASE_URI[] = {
 			"http://a/b/c/d;p?q",
 			"http://a/b/c/d;p?q=1/2",
 			"http://a/b/c/d;p=1/2?q",
