@@ -84,9 +84,12 @@ public:
 		TEST_ADD(UriSuite::testNormalizeCrash_Bug20080224)
 		TEST_ADD(UriSuite::testFilenameUriConversion)
 		TEST_ADD(UriSuite::testCrash_FreeUriMembers_Bug20080116)
+		TEST_ADD(UriSuite::testCrash_Report2418192)
+		TEST_ADD(UriSuite::testPervertedQueryString);
 		TEST_ADD(UriSuite::testCrash_MakeOwner_Bug20080207)
 		TEST_ADD(UriSuite::testQueryList)
 		TEST_ADD(UriSuite::testQueryListPair)
+		TEST_ADD(UriSuite::testFreeCrash_Bug20080827)
 	}
 
 private:
@@ -456,7 +459,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		stateA.uri = &uriA;
 
 		int res;
-		
+
 		res = uriParseUriA(&stateA, "http://user:21@host/");
 		TEST_ASSERT(URI_SUCCESS == res);
 		TEST_ASSERT(!memcmp(uriA.userInfo.first, "user:21", 7 * sizeof(char)));
@@ -465,15 +468,19 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		TEST_ASSERT(uriA.hostText.afterLast - uriA.hostText.first == 4);
 		TEST_ASSERT(uriA.portText.first == NULL);
 		TEST_ASSERT(uriA.portText.afterLast == NULL);
+		uriFreeUriMembersA(&uriA);
 
 		res = uriParseUriA(&stateA, "http://user:1234@192.168.0.1:1234/foo.com");
 		TEST_ASSERT(URI_SUCCESS == res);
+		uriFreeUriMembersA(&uriA);
 
 		res = uriParseUriA(&stateA, "http://moo:21@moo:21@moo/");
 		TEST_ASSERT(URI_ERROR_SYNTAX == res);
+		uriFreeUriMembersA(&uriA);
 
 		res = uriParseUriA(&stateA, "http://moo:21@moo:21@moo:21/");
 		TEST_ASSERT(URI_ERROR_SYNTAX == res);
+		uriFreeUriMembersA(&uriA);
 	}
 
 	void testUriUserInfoHostPort3() {
@@ -701,12 +708,14 @@ Rule                                | Example | hostSet | absPath | emptySeg
 
 	bool testUnescapingHelper(const wchar_t * input, const wchar_t * output,
 			bool plusToSpace = false, UriBreakConversion breakConversion = URI_BR_DONT_TOUCH) {
-		wchar_t * working = static_cast<URI_CHAR *>(malloc((URI_STRLEN(input) + 1) * sizeof(URI_CHAR)));
+		wchar_t * working = new wchar_t[URI_STRLEN(input) + 1];
 		wcscpy(working, input);
 		const wchar_t * newTermZero = uriUnescapeInPlaceExW(working,
 				plusToSpace ? URI_TRUE : URI_FALSE, breakConversion);
-		return ((newTermZero == working + wcslen(output))
+		const bool success = ((newTermZero == working + wcslen(output))
 				&& !wcscmp(working, output));
+		delete[] working;
+		return success;
 	}
 
 	void testUnescaping() {
@@ -805,6 +814,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		stateW.uri = &baseUri;
 		int res = uriParseUriW(&stateW, base);
 		if (res != 0) {
+			uriFreeUriMembersW(&baseUri);
 			return false;
 		}
 
@@ -814,6 +824,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		res = uriParseUriW(&stateW, rel);
 		if (res != 0) {
 			uriFreeUriMembersW(&baseUri);
+			uriFreeUriMembersW(&relUri);
 			return false;
 		}
 
@@ -824,6 +835,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		if (res != 0) {
 			uriFreeUriMembersW(&baseUri);
 			uriFreeUriMembersW(&relUri);
+			uriFreeUriMembersW(&expectedUri);
 			return false;
 		}
 
@@ -833,6 +845,8 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		if (res != 0) {
 			uriFreeUriMembersW(&baseUri);
 			uriFreeUriMembersW(&relUri);
+			uriFreeUriMembersW(&expectedUri);
+			uriFreeUriMembersW(&transformedUri);
 			return false;
 		}
 
@@ -842,12 +856,15 @@ Rule                                | Example | hostSet | absPath | emptySeg
 			wchar_t expectedUriText[1024 * 8];
 			uriToStringW(transformedUriText, &transformedUri, 1024 * 8, NULL);
 			uriToStringW(expectedUriText, &expectedUri, 1024 * 8, NULL);
+#ifdef HAVE_WPRINTF
 			wprintf(L"\n\n\nExpected: \"%s\"\nReceived: \"%s\"\n\n\n", expectedUriText, transformedUriText);
+#endif
 		}
 
 		uriFreeUriMembersW(&baseUri);
 		uriFreeUriMembersW(&relUri);
 		uriFreeUriMembersW(&expectedUri);
+		uriFreeUriMembersW(&transformedUri);
 		return equal;
 	}
 
@@ -937,7 +954,9 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		// Compare
 		bool equals = (0 == wcscmp(shouldbeTheSame, text));
 		if (!equals) {
+#ifdef HAVE_WPRINTF
 			wprintf(L"\n\n\nExpected: \"%s\"\nReceived: \"%s\"\n\n\n", text, shouldbeTheSame);
+#endif
 		}
 
 		// Back to string, _exact_ limit
@@ -1305,7 +1324,9 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		} else {
 			uriWindowsFilenameToUriStringW(filename, uriBuffer);
 		}
+#ifdef HAVE_WPRINTF
 		// wprintf(L"1 [%s][%s]\n", uriBuffer, uriString);
+#endif
 		TEST_ASSERT(!wcscmp(uriBuffer, uriString));
 		delete [] uriBuffer;
 
@@ -1317,7 +1338,9 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		} else {
 			uriUriStringToWindowsFilenameW(uriString, filenameBuffer);
 		}
+#ifdef HAVE_WPRINTF
 		// wprintf(L"2 [%s][%s]\n", filenameBuffer, filename);
+#endif
 		TEST_ASSERT(!wcscmp(filenameBuffer, filename));
 		delete [] filenameBuffer;
 	}
@@ -1350,6 +1373,34 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		TEST_ASSERT(true);
 	}
 
+	void testCrash_Report2418192() {
+		// Testcase by Harvey Vrsalovic
+		helperTestQueryString("http://svcs.cnn.com/weather/wrapper.jsp?&csiID=csi1", 1);
+	}
+
+	void testPervertedQueryString() {
+		helperTestQueryString("http://example.org/?&&=&&&=&&&&==&===&====", 5);
+	}
+
+	void helperTestQueryString(char const * uriString, int pairsExpected) {
+		UriParserStateA state;
+		UriUriA uri;
+		state.uri = &uri;
+		int res = uriParseUriA(&state, uriString);
+		TEST_ASSERT(res == URI_SUCCESS);
+
+		UriQueryListA * queryList = NULL;
+		int itemCount = 0;
+
+		res = uriDissectQueryMallocA(&queryList, &itemCount,
+				uri.query.first, uri.query.afterLast);
+		TEST_ASSERT(res == URI_SUCCESS);
+		TEST_ASSERT(queryList != NULL);
+		TEST_ASSERT(itemCount == pairsExpected);
+		uriFreeQueryListA(queryList);
+		uriFreeUriMembersA(&uri);
+	}
+
 	void testCrash_MakeOwner_Bug20080207() {
 		// Testcase by Adrian Manrique
 		UriParserStateA state;
@@ -1368,7 +1419,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 
 	void testQueryListHelper(const wchar_t * input, int expectedItemCount) {
 		int res;
-		
+
 		UriBool spacePlusConversion = URI_TRUE;
 		UriBool normalizeBreaks = URI_FALSE;
 		UriBreakConversion breakConversion = URI_BR_DONT_TOUCH;
@@ -1382,9 +1433,9 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		TEST_ASSERT((queryList == NULL) == (expectedItemCount == 0));
 
 		if (expectedItemCount != 0) {
-			// First 
+			// First
 			int charsRequired;
-			res = uriComposeQueryCharsRequiredExW(queryList, &charsRequired, spacePlusConversion, 
+			res = uriComposeQueryCharsRequiredExW(queryList, &charsRequired, spacePlusConversion,
 					normalizeBreaks);
 			TEST_ASSERT(res == URI_SUCCESS);
 			TEST_ASSERT(charsRequired >= (int)wcslen(input));
@@ -1425,7 +1476,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		int res;
 		UriQueryListA * queryList;
 		int itemCount;
-		
+
 		res = uriDissectQueryMallocA(&queryList, &itemCount, pair, pair + strlen(pair));
 		TEST_ASSERT(res == URI_SUCCESS);
 		TEST_ASSERT(queryList != NULL);
@@ -1439,6 +1490,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		TEST_ASSERT(recomposed != NULL);
 		TEST_ASSERT(!strcmp(recomposed, (fixed != NULL) ? fixed : pair));
 		free(recomposed);
+		uriFreeQueryListA(queryList);
 	}
 
 	void testQueryListPair() {
@@ -1447,6 +1499,32 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		testQueryListPairHelper("one=two=three=four", "one", "two=three=four", "one=two%3Dthree%3Dfour");
 	}
 
+	void testFreeCrash_Bug20080827() {
+		char const * const sourceUri = "abc";
+		char const * const baseUri = "http://www.example.org/";
+
+		int res;
+		UriParserStateA state;
+		UriUriA absoluteDest;
+		UriUriA relativeSource;
+		UriUriA absoluteBase;
+
+		state.uri = &relativeSource;
+		res = uriParseUriA(&state, sourceUri);
+		TEST_ASSERT(res == URI_SUCCESS);
+
+		state.uri = &absoluteBase;
+		res = uriParseUriA(&state, baseUri);
+		TEST_ASSERT(res == URI_SUCCESS);
+
+		res = uriRemoveBaseUriA(&absoluteDest, &relativeSource, &absoluteBase, URI_FALSE);
+		TEST_ASSERT(res == URI_ERROR_REMOVEBASE_REL_SOURCE);
+
+		uriFreeUriMembersA(&relativeSource);
+		uriFreeUriMembersA(&absoluteBase);
+		uriFreeUriMembersA(&absoluteDest); // Crashed here
+	}
+
 };
 
 
